﻿#version 330
uniform sampler2D tex;
uniform sampler2D end;
uniform float iTime;
uniform vec2 iResolution;
out vec4 fragColor;

vec2 uv;

const vec2 ch_size  = vec2(1.0, 2.0);              // character size (X,Y)
const vec2 ch_space = ch_size + vec2(1.0, 1.0);    // character distance Vector(X,Y)
const vec2 ch_start = vec2 (ch_space.x * -6., 2.); // start position
      vec2 ch_pos   = vec2 (0.0, 0.0);             // character position(X,Y)
      vec3 ch_color = vec3 (0.0, 0.5, 2.5);        // character color (R,G,B)
const vec3 bg_color = vec3 (0.0, 0.0, 0.0);        // background color (R,G,B)

#define REPEAT_SIGN false // True/False; True=Multiple, False=Single

/* 16 segment display...Akin to LED Display.

Segment bit positions:
  __2__ __1__
 |\    |    /|
 | \   |   / |
 3  11 10 9  0
 |   \ | /   |
 |    \|/    |
  _12__ __8__
 |           |
 |    /|\    |
 4   / | \   7
 | 13 14  15 |
 | /   |   \ |
  __5__|__6__

15 12 11 8 7  4 3  0
 |  | |  | |  | |  |
 0000 0000 0000 0000

example: letter A

   12    8 7  4 3210
    |    | |  | ||||
 0001 0001 1001 1111

 binary to hex -> 0x119F
*/

#define n0 ddigit(0x22FF);
#define n1 ddigit(0x0281);
#define n2 ddigit(0x1177);
#define n3 ddigit(0x11E7);
#define n4 ddigit(0x1189);
#define n5 ddigit(0x11EE);
#define n6 ddigit(0x11FE);
#define n7 ddigit(0x0087);
#define n8 ddigit(0x11FF);
#define n9 ddigit(0x11EF);

#define A ddigit(0x119F);
#define B ddigit(0x927E);
#define C ddigit(0x007E);
#define D ddigit(0x44E7);
#define E ddigit(0x107E);
#define F ddigit(0x101E);
#define G ddigit(0x807E);
#define H ddigit(0x1199);
#define I ddigit(0x4466);
#define J ddigit(0x4436);
#define K ddigit(0x9218);
#define L ddigit(0x0078);
#define M ddigit(0x0A99);
#define N ddigit(0x8899);
#define O ddigit(0x00FF);
#define P ddigit(0x111F);
#define Q ddigit(0x80FF);
#define R ddigit(0x911F);
#define S ddigit(0x8866);
#define T ddigit(0x4406);
#define U ddigit(0x00F9);
#define V ddigit(0x2218);
#define W ddigit(0xA099);
#define X ddigit(0xAA00);
#define Y ddigit(0x4A00);
#define Z ddigit(0x2266);
#define _ ch_pos.x += ch_space.x;
#define s_dot     ddigit(0);
#define s_minus   ddigit(0x1100);
#define s_plus    ddigit(0x5500);
#define s_greater ddigit(0x2800);
#define s_less    ddigit(0x8200);
#define s_sqrt    ddigit(0x0C02);
#define nl1 ch_pos = ch_start;  ch_pos.y -= 3.0;
#define nl2 ch_pos = ch_start;  ch_pos.y -= 6.0;
#define nl3 ch_pos = ch_start;	ch_pos.y -= 9.0;
#define nl4 ch_pos = ch_start;	ch_pos.y -= 12;
#define nl5 ch_pos = ch_start;	ch_pos.y -= 15;

float dseg(vec2 p0, vec2 p1)
{
	vec2 dir = normalize(p1 - p0);
	vec2 cp = (uv - ch_pos - p0) * mat2(dir.x, dir.y,-dir.y, dir.x);
	return distance(cp, clamp(cp, vec2(0), vec2(distance(p0, p1), 0)));   
}

bool bit(int n, int b)
{
	return mod(floor(float(n) / exp2(floor(float(b)))), 2.0) != 0.0;
}

float d = 1e6;

void ddigit(int n)
{
	float v = 1e6;	
	vec2 cp = uv - ch_pos;
	if (n == 0)     v = min(v, dseg(vec2(-0.405, -1.000), vec2(-0.500, -1.000)));
	if (bit(n,  0)) v = min(v, dseg(vec2( 0.500,  0.063), vec2( 0.500,  0.937)));
	if (bit(n,  1)) v = min(v, dseg(vec2( 0.438,  1.000), vec2( 0.063,  1.000)));
	if (bit(n,  2)) v = min(v, dseg(vec2(-0.063,  1.000), vec2(-0.438,  1.000)));
	if (bit(n,  3)) v = min(v, dseg(vec2(-0.500,  0.937), vec2(-0.500,  0.062)));
	if (bit(n,  4)) v = min(v, dseg(vec2(-0.500, -0.063), vec2(-0.500, -0.938)));
	if (bit(n,  5)) v = min(v, dseg(vec2(-0.438, -1.000), vec2(-0.063, -1.000)));
	if (bit(n,  6)) v = min(v, dseg(vec2( 0.063, -1.000), vec2( 0.438, -1.000)));
	if (bit(n,  7)) v = min(v, dseg(vec2( 0.500, -0.938), vec2( 0.500, -0.063)));
	if (bit(n,  8)) v = min(v, dseg(vec2( 0.063,  0.000), vec2( 0.438, -0.000)));
	if (bit(n,  9)) v = min(v, dseg(vec2( 0.063,  0.063), vec2( 0.438,  0.938)));
	if (bit(n, 10)) v = min(v, dseg(vec2( 0.000,  0.063), vec2( 0.000,  0.937)));
	if (bit(n, 11)) v = min(v, dseg(vec2(-0.063,  0.063), vec2(-0.438,  0.938)));
	if (bit(n, 12)) v = min(v, dseg(vec2(-0.438,  0.000), vec2(-0.063, -0.000)));
	if (bit(n, 13)) v = min(v, dseg(vec2(-0.063, -0.063), vec2(-0.438, -0.938)));
	if (bit(n, 14)) v = min(v, dseg(vec2( 0.000, -0.938), vec2( 0.000, -0.063)));
	if (bit(n, 15)) v = min(v, dseg(vec2( 0.063, -0.063), vec2( 0.438, -0.938)));
	ch_pos.x += ch_space.x;
	d = min(d, v);
}


vec3 jittering(in sampler2D textu, in vec2 uv){
	float amount = 0.;

	amount = (1.0 + sin(iTime*6.0)) * 0.5;
	amount *= 1.0 + sin(iTime*16.0) * 0.5;
	amount *= 1.0 + sin(iTime*19.0) * 0.5;
	amount *= 1.0 + sin(iTime*27.0) * 0.5;
	amount = pow(amount, 3.0);

	amount *= 0.05;

	vec3 col;
	col.r = texture(textu, vec2(uv.x+amount, uv.y + amount)).r;
	col.g = texture(textu, uv).g;
	col.b = texture(textu, vec2(uv.x - amount , uv.y - amount)).b;

	return col *= (1.0 - amount * 0.5);
}

void endText(in vec2 aspect){
		float amount = 0.;

		amount = (1.0 + sin(iTime*6.0)) * 0.5;
		amount *= 1.0 + sin(iTime*16.0) * 0.5;
		amount *= 1.0 + sin(iTime*19.0) * 0.5;
		amount *= 1.0 + sin(iTime*27.0) * 0.5;
		amount = pow(amount, 3.0);

		amount *= 0.05;
		uv = ( gl_FragCoord.xy / iResolution.y ) - aspect / 2.0;
		uv *= 20.0;    //  set zoom size
		uv.x += amount;
		uv.y -= amount;

		uv = ( gl_FragCoord.xy / iResolution.y ) - aspect / 2.0;
		uv *= 20.0;    //  set zoom size
		if (REPEAT_SIGN)
		  uv = -14.0 + mod(1.8*(uv-1.0),ch_space*vec2(16.,6.5));     //  set zoom size

		ch_pos = ch_start;// + vec2(sin(iTime),0.0);  // set start position
		D E M O nl1
		O N nl2
		L O P P U nl3
		T A P P A K A A nl4
		I T S E N N E
}

void otherText(in vec2 aspect, out float amount){

		amount = 0.;

		amount = (1.0 + sin(iTime*6.0)) * 0.5;
		amount *= 1.0 + sin(iTime*16.0) * 0.5;
		amount *= 1.0 + sin(iTime*19.0) * 0.5;
		amount *= 1.0 + sin(iTime*27.0) * 0.5;
		amount = pow(amount, 3.0);

		amount *= 0.05;
		uv = ( gl_FragCoord.xy / iResolution.y ) - aspect / 2.0;
		uv *= 20.0;    //  set zoom size
		uv.x += amount;
		uv.y -= amount;
		if (REPEAT_SIGN)
		  uv = -14.0 + mod(1.8*(uv-1.0),ch_space*vec2(16.,6.5));     //  set zoom size

		ch_pos = ch_start;// + vec2(sin(iTime),0.0);  // set start position
}

void main(){
	
	uv = gl_FragCoord.xy / iResolution.xy;
	vec2 aspect = iResolution.xy / gl_FragCoord.xy;
	
	vec3 col = jittering(tex, uv);
	float amount;
	vec4 endColor = vec4(col, 1.);

	if(iTime <= 2.){
		otherText(aspect, amount);
		M O K K nl1
		D E M
		vec3 color = mix(ch_color, bg_color, 1. - (0.08 / d));
		endColor = vec4(mix(color, col, iTime - 1.), 1.);
	}
	if(iTime >= 5. && iTime <= 7.){
		otherText(aspect, amount);
		N O nl1
		S L E E P
		vec3 color = mix(ch_color, bg_color, 1. - (0.08 / d));
		color *= (1.0 - amount * 0.5);
		endColor = vec4(mix(color, col, 0.2), 1.);
	}else if(iTime >= 10. && iTime <= 13.){
		otherText(aspect, amount);
		O N L Y nl1
		T H E nl2
		O N E
		vec3 color = mix(ch_color, bg_color, 1. - (0.08 / d));
		endColor = vec4(mix(color, col, 0.2), 1.);
	}
	if(iTime >= 18. && iTime <= 19.){
		vec4 endCol = texture(end, uv);
		col = mix(col, vec3(0.), (iTime - 18.));
		endColor = vec4(col, 1.);
	} if(iTime >= 19. && iTime <= 23.){
		otherText(aspect, amount);
		M O I M A N
		vec3 color = mix(ch_color, bg_color, 1. - (0.08 / d));
		vec3 col = jittering(end, gl_FragCoord.xy / iResolution.xy);

		endColor = vec4(mix(color, col, clamp(iTime - 22., 0., 1.)), 1.);
	}if(iTime >= 23. && iTime <= 27.){
		endColor = vec4(jittering(end, uv), 1.);
		//fragColor = texture(end, uv);
	}else if(iTime >= 27.){
		endText(aspect);
		vec4 texColor = vec4(jittering(end, gl_FragCoord.xy / iResolution.xy), 1.);
		vec3 color = mix(ch_color, texColor.xyz, 1.0- (0.08 / d));  // shading
		endColor = mix(vec4(color, 1.), texColor, 0.5);
	}

	fragColor = endColor;
}