﻿#version 330
uniform vec2 iResolution;
uniform float iTime;
out vec4 fragColor;

const float PI = 3.141592653589793284;

float Hash( float n )
{
    return fract(sin(n)*43758.5453123);
}

mat4 rotY(float angle){
    float c = cos(angle);
    float s = sin(angle);
    return mat4(
        vec4(c, 0, s, 0),
        vec4(0, 1, 0, 0),
        vec4(-s, 0, c, 0),
        vec4(0, 0, 0, 1)
    );
}

mat3 rotY3(float angle){
    float c = cos(angle);
    float s = sin(angle);
    return mat3(
        c, 0., s,
        0., 1., 0.,
        -s, 0., c);
}
float rand(vec2 co){
    return fract(sin(dot(co.xy ,vec2(12.9898,78.233))) * 43758.5453);
}

float hash (float n)
{
    return fract(sin(n)*43758.5453);
}
float noise (in vec3 x)
{
    vec3 p = floor(x);
    vec3 f = fract(x);
    f = f*f*(3.0-2.0*f);
    float n = p.x + p.y*57.0 + 113.0*p.z;
    float res = mix(mix(mix( hash(n+  0.0), hash(n+  1.0),f.x),
                        mix( hash(n+ 57.0), hash(n+ 58.0),f.x),f.y),
                    mix(mix( hash(n+113.0), hash(n+114.0),f.x),
                        mix( hash(n+170.0), hash(n+171.0),f.x),f.y),f.z);
    return res;
}

mat3 m = mat3( 0.00,  0.80,  0.60,
              -0.80,  0.36, -0.48,
              -0.60, -0.48,  0.64 );
float fbm (vec3 p)
{
    float f;
    f  = 0.5000*noise( p ); p = m*p*2.02;
    f += 0.2500*noise( p ); p = m*p*2.03;
    f += 0.1250*noise( p ); //p = m*p*2.01;
    f += 0.0625*noise( p );
    return f;
}


vec2 opU(vec2 d1, vec2 d2)
{
    return (d1.x < d2.x) ? d1 : d2;
}


vec2 opS(vec2 d1, vec2 d2)
{
    return (-d1.x > d2.x) ? d1*vec2(-1,1) : d2;
}


vec2 sdBox(vec3 pos, vec3 size, float id)
{
    return vec2(length(max(abs(pos) - size, 0.0)), id);
}

vec2 sdSphere(vec3 p, float s, float id)
{
  return vec2(length(p) - s, id);
}

float smin( float a, float b, float k ) {
    float res = exp( -k*a ) + exp( -k*b );
    return -log( res )/k;
}

float cube(in vec3 p, in vec3 b) {
  vec3 d = abs(p) - b;
  return min(max(d.x,max(d.y,d.z)),0.0) + length(max(d,0.0));
}

float sphere(in vec3 p, in float s) {
  return length(p)-s;
}
float plane( vec3 p, vec4 n ) {
  // n must be normalized
  return dot(p,n.xyz) + n.w;
}
float plane(vec3 p){
    return p.y;   
}

vec3 cPos = vec3(1., -1., 1.);

vec2 hash22(vec2 p)
{
    p  = fract(p * vec2(5.3983, 5.4427));
    p += dot(p.yx, p.xy +  vec2(21.5351, 14.3137));
    return fract(vec2(p.x * p.y * 95.4337, p.x * p.y * 97.597));
}

float height (in vec2 p){
    p *= 0.2;
    return sin(p.y) *  2.0 + cos(p.x) * 0.4;
}

float vine(vec3 p, in float c, in float h)
{
    p.y += sin(p.z*0.2625)*2.5;
    p.x += cos(p.z*0.1575)*3.;
    vec2 q = vec2(mod(p.x, c)-c/2., p.y);
    return length(q) - h -sin(p.z*2.+sin(p.x*7.)*0.5+iTime*0.5)*0.13;
}


vec2 Scene(vec3 p){
    vec3 org = p;
    p.y += height(p.zx);
    vec2 res = vec2(plane(vec3(p.x, p.y + 5., p.z)), 1.);
    p = org;
    res = opU(res, vec2(vine(org + vec3(1.8, 0., 0) + normalize(cPos - p) - length(normalize(cPos - p)), 15., 0.8), 1.));
    res = opU(res, vec2(vine(vec3(p.z, sin(p.y * PI) - cos(p.x * PI), p.x) + vec3(0., 0, 17.), 20., 0.75), 1.));
    res = opU(res, vec2(vine(p - (vec3(cPos)), 13., 0.7), 1.));
    //res = opU(vec2(cube(p, vec3(1.0)), 1.), res);

    res = opU(res, sdSphere(p, 1.0, 2.));
    //res = opU(res, vec2(plane(vec3(org.x, org.y + 5., org.z)), 1.));

    //res = opU(res, vec2(cube(p, vec3(1.0)), 1.));

    return res;
}

vec3 sceneNormals(vec3 p){
    vec3 e = vec3(0.001, 0., 0.);
    return normalize(
        vec3(
            Scene(p + e.xyy).x - Scene(p - e.xyy).x,
            Scene(p + e.yxz).x - Scene(p - e.yxy).x,
            Scene(p + e.yyx).x - Scene(p - e.yyx).x
            )
        );
}

vec3 trace(in vec3 p, in vec3 dir, in float tmin, in float tmax, out float ao){
    vec2 rest = vec2(-1.0);
    float t = tmin;
    int item = 0;
    for(int i = 0; i < 100; i++){
        rest = Scene(p + dir * t);
        if(rest.x < 0.001 || t > tmax) break;
        t += rest.x;
        ao = 1.0 - float(i) / 128.;

    }
    vec3 res = vec3(rest.xy, t);
    return res;
    
}

vec3 specularcol(vec3 pos, vec3 normal, vec3 light){
    vec3 toEye = normalize(vec3(0.) - pos);
    vec3 lightRef = normalize(reflect(-light, normal));
    float spec = max(0., pow(dot(lightRef, toEye), 64.));
    return vec3(1.0) * max(spec, 0.);
}

float Traceshadow(in vec3 p, in vec3 dir, float tmin, float tmax, float k){
    float rest = 1.;
    float t = tmin;
    int item = 0;
    for(int i = 0; i < 128; i++){
        float h = Scene(p + dir * t).x;
        if(h < 0.001)
            return 0.;
        rest = min(rest, k * h / t);
        t += rest;
        
    }
    return rest;
}

vec3 getLightPos(){
    return vec3(sin(5. +iTime * PI), 5., 4.);
}

vec3 sphereMaterial(in vec3 p, in vec3 nor){
    
    float a = atan(p.y, p.x);
    float r = length(p);

    vec3 baseColor = vec3(1., 0.0, 0.0);
    vec3 uv = 5.0 * p;

    float c = fbm(p * 4.);
    float y = smoothstep(0.0, 1.0, uv.x);

    return mix(baseColor, uv, y);

}

vec3 scenerender(vec3 p, vec3 dir, vec2 tmima, out float dist){
    vec3 res = vec3(0.34);
    float ao;
    vec3 t = trace(p, dir, tmima.x, tmima.y, ao);
    dist = t.z;
    if(t.x < 1.0 ){
        vec3 col = vec3(0.);
        vec3 lightPos = getLightPos();
        vec3 pos = p + t.z * dir;

        vec3 lightDir = normalize(vec3(1.0, 1.0, -0.3));
		float sundot = clamp(dot(dir, lightDir), 0., 1.);
        //lightPos = vec3(lightPos.x * sin(iTime), lightPos.y + 5. * cos(iTime), lightPos.z);
        vec3 normals = sceneNormals(pos);
        float a;
        float shadow = Traceshadow(pos + normals * 0.1, lightDir, tmima.x, tmima.y, 32.);
        float light = 0.4;
        vec3 specColor = (pos, normals, vec3(1., 1., 0.));
        vec3 L = lightPos - pos;
        vec3 lnorm = L / length(L);
        vec3 fogb = mix(vec3(0.7, 0.8, 0.8) * 0.3, vec3(1., 1., .77), pow(dot(dir, lightDir) + 1.2, 2.5) * .15);
        fogb *= clamp(dir.y * 0.6 + .6, 0., 1.);
        col = fogb;
        if(t.y == 1.){
            col = vec3(0.05, 0.1, 1.0);   
        }else if (t.y == 2.){
            col = sphereMaterial(p, normals);
			float f = smoothstep(0.1, 1., fbm(pos * 4.));
			col = mix(col, vec3(0.6, 0.3, 0.), f);
        }else{
            col = vec3(0.);
        }
        
        if(shadow >= 0.1)
            light += max(dot(normals, lnorm) * 0.5, 0.0);
        col += specColor;
        vec3 posi = p + dir * t.z;
        res = col * light;
        res = mix(res, fogb, smoothstep(tmima.y - 7., tmima.y, dist));
        res *= ao;
    }
    return res;
}



vec3 rayDir(float fov, vec2 size, vec2 frag){
    vec2 xy = frag - size / 2.0;
    float z = size.y / tan(radians(fov) / 2.0);
    return normalize(vec3(xy, -z));
}

mat4 viewMatrix(vec3 eye, vec3 center, vec3 up){
    vec3 f = normalize(center - eye);
    vec3 s = normalize(cross(f, up));
    vec3 u = cross(s, f);
    return mat4(
        vec4(s, 0.0),
        vec4(u, 0.0),
        vec4(-f, 0.0),
        vec4(0.0, 0.0, 0.0, 1.0)
    );
}

float tri(in float x){return abs(fract(x)-.5);}
vec3 tri3(in vec3 p){return vec3( tri(p.z+tri(p.y*1.)), tri(p.z+tri(p.x*1.)), tri(p.y+tri(p.x*1.)));}
                                 
mat2 m2 = mat2(0.970,  0.242, -0.242,  0.970);

float triNoise3d(in vec3 p, in float spd)
{
    float z=1.4;
    float rz = 0.;
    vec3 bp = p;
    for (float i=0.; i<=3.; i++ )
    {
        vec3 dg = tri3(bp*2.);
        p += (dg+iTime*spd);

        bp *= 1.8;
        z *= 1.5;
        p *= 1.2;
        //p.xz*= m2;
        
        rz+= (tri(p.z+tri(p.x+tri(p.y))))/z;
        bp += 0.14;
    }
    return rz;
}

float fogmap(in vec3 p, in float d)
{
    p.x += iTime*1.5;
    p.z += sin(p.x*.5);
    return triNoise3d(p*2.2/(d+20.),0.2)*(1.-smoothstep(0.,.7,p.y));
}

vec3 fog(in vec3 col, in vec3 ro, in vec3 rd, in float mt)
{
    float d = .5;
    for(int i=0; i<7; i++)
    {
        vec3  pos = ro + rd*d;
        float rz = fogmap(pos, d);
        float grd =  clamp((rz - fogmap(pos+.8-float(i)*0.1,d))*3., 0.1, 1. );
        vec3 col2 = (vec3(.1,0.8,.5)*.5 + .5*vec3(.5, .8, 1.)*(1.7-grd))*0.55;
        col = mix(col,col2,clamp(rz*smoothstep(d-0.4,d+2.+d*.75,mt),0.,1.) );
        d *= 1.5+0.3;
        if (d>mt)break;
    }
    return col;
}

void mainImage()
{
	vec2 uv = gl_FragCoord.xy / iResolution.xy;
    vec3 col = vec3(0.0);
    vec3 eye = vec3(0.0, 0.0, 10.);
    if(iTime >= 5.){
        eye += normalize(eye - vec3(0.)) * (iTime - 5.) * 2.;
    }
    vec3 viewDir = rayDir(60., iResolution.xy, gl_FragCoord.xy);
    mat4 Temp = viewMatrix(eye, vec3(0.), vec3(0.0, 1.0, 0.0));
    //eye -= (Temp * vec4(viewDir, 0.0)).xyz * iTime;
    float dist = 0.;
    mat4 viewToWorld = viewMatrix(eye, vec3(0.0, 0., 0.), vec3(0.0, 1.0, 0.0));
    vec3 worldDir = (viewToWorld * vec4(viewDir, 0.0)).xyz;
    //scene
    col = scenerender(eye, worldDir, vec2(0., 128.), dist);
    col = fog(col, eye, worldDir, dist);
    col = pow(col, vec3(2.2));
    //col *= length(uv);
    if(iTime <= 2.0){
        col = mix(vec3(0.0), col, iTime - 1.);
    }
    fragColor = vec4(col, 1.0);
}

void main(){
	mainImage();
}